import pandas as pd
import networkx as nx
from collections import Counter

df = pd.read_csv("../outputs/projection_artist.txt", sep = "\t")
G = nx.from_pandas_edgelist(df, source = "src", target = "trg", edge_attr = True)

artists_stats = {}
artists_stats["nodes"] = len(G.nodes)
artists_stats["edges"] = len(G.edges)
artists_stats["avg_deg"] = 2 * len(G.edges) / len(G.nodes)
artists_stats["density"] = len(G.edges) / (len(G.nodes) * (len(G.nodes) - 1) / 2)
artists_stats["cc"] = nx.transitivity(G)
artists_stats["modularity"] = nx.community.modularity(G, nx.community.louvain_communities(G))

ccdf = Counter(dict(nx.degree(G)).values())
ccdf = pd.DataFrame(data = [(x, ccdf[x]) for x in ccdf], columns = ("k", "count")).sort_values(by = "k", ascending = False)
ccdf["cumsum"] = ccdf["count"].cumsum()
ccdf["cumsum"] /= ccdf["cumsum"].max()
ccdf.to_csv("../outputs/fig5a.csv", sep = "\t", index = False)

artists_degree = dict(sorted(dict(G.degree).items(), key = lambda item: item[1], reverse = True))
# While this code will reproduce the results in the paper, it is very slow and I suggest estimating these centralities with Cytoscape or another library
#artists_betweenness = dict(nx.betweenness_centrality(G))
#artists_closeness = dict(nx.closeness_centrality(G))

df = pd.read_csv("../outputs/projection_band.txt", sep = "\t")
G = nx.from_pandas_edgelist(df, source = "src", target = "trg", edge_attr = True)

band_stats = {}
band_stats["nodes"] = len(G.nodes)
band_stats["edges"] = len(G.edges)
band_stats["avg_deg"] = 2 * len(G.edges) / len(G.nodes)
band_stats["density"] = len(G.edges) / (len(G.nodes) * (len(G.nodes) - 1) / 2)
band_stats["cc"] = nx.transitivity(G)
band_stats["modularity"] = nx.community.modularity(G, nx.community.louvain_communities(G))

ccdf = Counter(dict(nx.degree(G)).values())
ccdf = pd.DataFrame(data = [(x, ccdf[x]) for x in ccdf], columns = ("k", "count")).sort_values(by = "k", ascending = False)
ccdf["cumsum"] = ccdf["count"].cumsum()
ccdf["cumsum"] /= ccdf["cumsum"].max()
ccdf.to_csv("../outputs/fig5b.csv", sep = "\t", index = False)

bands_degree = dict(sorted(dict(G.degree).items(), key = lambda item: item[1], reverse = True))
# While this code will reproduce the results in the paper, it is very slow and I suggest estimating these centralities with Cytoscape or another library
#bands_betweenness = dict(nx.betweenness_centrality(G))
#bands_closeness = dict(nx.closeness_centrality(G))

print("\\begin{table}")
print("\\centering")
print("\\begin{tabular}{l|rr}")
print("Variable & Artist & Band\\\\")
print("\\hline")
print(f"\\# Nodes & {artists_stats['nodes']} & {band_stats['nodes']}\\\\")
print(f"\\# Edges & {artists_stats['edges']} & {band_stats['edges']}\\\\")
print(f"Avg Deg & {artists_stats['avg_deg']:.1f} & {band_stats['avg_deg']:.1f}\\\\")
print(f"Density & {artists_stats['density']:.4f} & {band_stats['density']:.4f}\\\\")
print(f"Clustering & {artists_stats['cc']:.4f} & {band_stats['cc']:.4f}\\\\")
print(f"Modularity & {artists_stats['modularity']:.4f} & {band_stats['modularity']:.4f}\\\\")
print("\\end{tabular}")
print("\\caption{Summary statistics for the projected networks.}")
print("\\label{tab:summary-projection}")
print("\\end{table}")


