import pandas as pd
import networkx as nx

G = pd.read_csv("../outputs/projection_band.txt", sep = "\t")
G = nx.from_pandas_edgelist(G, source = "src", target = "trg", edge_attr = True)

df = pd.read_csv("../data/years.tsv", sep = "\t").set_index("id")
df = (df * df.columns.values.astype(int)).apply(lambda x: x[x > 0].mean(), axis = 1).reset_index().rename(columns = {0: "avgyear"})
df.to_csv("../outputs/fig6_nodeattrs.tsv", sep = "\t", index = False)

pd.qcut(df["avgyear"], q = 8)
nx.set_node_attributes(G, df.set_index("id")["avgyear"].to_dict(), name = "year")
year_assortativity = nx.numeric_assortativity_coefficient(G, "year")
print(f"Network assortativity of AVG year: {year_assortativity:.3f}")

df = pd.read_csv("../data/genres.tsv", sep = "\t").set_index("id")
df = (df.T / df.sum(axis = 1)).T.fillna(0.0)
dominant_genres = ["Pop", "Rock", "Electronic", "Hip Hop"]
df = df[dominant_genres].copy()
df = df.map(lambda x: x if x >= 0.05 else 0)
df.loc[df[dominant_genres].sum(axis = 1) > 0, "dominant"] = df.loc[df[dominant_genres].sum(axis = 1) > 0, dominant_genres].idxmax(axis = 1).values
df.loc[df[dominant_genres].sum(axis = 1) == 0, "dominant"] = "Other"
df.groupby(by = "dominant").size()
df = df.reset_index()
df[["id", "dominant"]].to_csv("../outputs/fig7_nodeattrs.tsv", sep = "\t", index = False)

nx.set_node_attributes(G, df.set_index("id")["dominant"].to_dict(), name = "genre")
genre_modularity = nx.community.modularity(G, list(df.groupby(by = "dominant")["id"].apply(list).values))
genre_assortativity = nx.attribute_assortativity_coefficient(G, "genre")

print(f"Modularity of the dominant genres: {genre_modularity:.3f}")
print(f"Network assortativity of dominant genres: {genre_assortativity:.3f}")

df = pd.read_csv("../data/regions.tsv", sep = "\t")
df = df.set_index("id")
regions = df.columns.to_numpy()
df = pd.DataFrame([regions[region] for region in df.eq(1).to_numpy()], index = df.index) 
nx.set_node_attributes(G, df[0].to_dict(), name = "region")
region_assortativity = nx.attribute_assortativity_coefficient(G, "region")

print(f"Network assortativity of origin region: {region_assortativity:.3f}")



