import torch
import numpy as np
import pandas as pd
import networkx as nx
from lib import torch_nvd as nvd

nodes = pd.read_csv("../data/years.tsv", sep = "\t").set_index("id")
edges = pd.read_csv("../outputs/projection_band.txt", sep = "\t")
edges = nx.from_pandas_edgelist(edges, source = "src", target = "trg", edge_attr = ["nij", "score"])

nodes /= nodes.sum()
nodes = nodes.reset_index()

nodelist = list(nodes["id"].sort_values())
nodemap = {nodelist[i]: i for i in range(len(nodelist))}
nodes["id"] = nodes["id"].map(nodemap)
nodes = nodes.sort_values(by = "id").set_index("id")
nodes = nodes.reindex(sorted(nodes.columns), axis = 1)
genremap = {i: nodes.columns[i] for i in range(len(nodes.columns))}
edges = nx.relabel_nodes(edges, nodemap)

tensor = nvd.make_tensor(edges, nodes)
Linv = nvd._Linv(tensor)
ER = nvd._er(tensor, Linv)

year_variances = pd.DataFrame(
   data = [(genremap[i], ((torch.outer(tensor.node_vects[:,i], tensor.node_vects[:,i]) * (ER ** 2)).sum() / 2).cpu().item()) for i in range(len(genremap))],
   columns = ("year", "variance")
)
year_variances.to_csv("../outputs/fig10.tsv", index = False, sep = "\t")
