import torch
import numpy as np
import pandas as pd
import networkx as nx
from lib import torch_nvd as nvd

def corr(tensor, i, j, W = None):
   numerator = (W * torch.outer(tensor.node_vects[:,i], tensor.node_vects[:,j])).sum()
   denominator_src = torch.sqrt((W * torch.outer(tensor.node_vects[:,i], tensor.node_vects[:,i])).sum())
   denominator_trg = torch.sqrt((W * torch.outer(tensor.node_vects[:,j], tensor.node_vects[:,j])).sum())
   return numerator / (denominator_src * denominator_trg)

genres = pd.read_csv("../data/genres.tsv", sep = "\t")#.set_index("id")
regions = pd.read_csv("../data/regions.tsv", sep = "\t")
edges = pd.read_csv("../outputs/projection_band.txt", sep = "\t")
edges = nx.from_pandas_edgelist(edges, source = "src", target = "trg", edge_attr = ["nij", "score"])

"""
genres_selected = (genres > 0).sum()
genres_selected = genres_selected[genres_selected > np.ceil(genres.shape[0] / 10)]
genres = genres[genres_selected.index].reset_index()
"""

nodelist = list(genres["id"].sort_values())
nodemap = {nodelist[i]: i for i in range(len(nodelist))}

genres["id"] = genres["id"].map(nodemap)
regions["id"] = regions["id"].map(nodemap)
genres = genres.sort_values(by = "id").set_index("id")
regions = regions.sort_values(by = "id").set_index("id")
genres = genres.reindex(sorted(genres.columns), axis = 1)
regions = regions.reindex(sorted(regions.columns), axis = 1)
genremap = {i: genres.columns[i] for i in range(len(genres.columns))}
regionmap = {i + len(genres.columns): regions.columns[i] for i in range(len(regions.columns))}

# Normalization not to let this being dominated by prolific bands
genres = (genres.T / genres.sum(axis = 1)).T.fillna(0.0)
# Normalization not to let this being dominated by popular genres
genres = genres / genres.sum(axis = 0)

regions /= regions.sum()
edges = nx.relabel_nodes(edges, nodemap)

tensor = nvd.make_tensor(edges, genres.join(regions))
Linv = nvd._Linv(tensor)
ER = nvd._er(tensor, Linv)
W = 1 / torch.exp(ER)

correlation_matrix = torch.zeros(genres.shape[1], regions.shape[1], device = "cuda")
for i in range(genres.shape[1]):
   for j in range(regions.shape[1]):
      correlation_matrix[i, j] = corr(tensor, i, j + genres.shape[1], W = W)

correlation_matrix = pd.DataFrame(data = correlation_matrix.cpu().numpy(), index = genres.columns, columns = regions.columns)
genre2region = correlation_matrix.T.idxmax().sort_values()

print("\\begin{table}")
print("\\centering")
print("\\begin{tabular}{l|r}")
print("Region & Genre\\\\")
print("\\hline")

for region in genre2region.drop_duplicates():
   print(f"{region.title()} & {correlation_matrix.loc[genre2region[genre2region == region].index, region].idxmax()}\\\\")

print("\\end{tabular}")
print("\\caption{The most correlated genre for a region, focusing on the genres which have their strongest correlation with that given region.}")
print("\\label{tab:genre-region-correlations}")
print("\\end{table}")
