import random
import numpy as np
import pandas as pd
import networkx as nx
from scipy.spatial.distance import cosine

edges = pd.read_csv("../outputs/projection_band_full.txt", sep = "\t")
genres = pd.read_csv("../data/genres.tsv", sep = "\t").set_index("id")
regions = pd.read_csv("../data/regions.tsv", sep = "\t").set_index("id")
years = pd.read_csv("../data/years.tsv", sep = "\t").set_index("id")

# Normalization not to let this being dominated by prolific bands
genres = (genres.T / genres.sum(axis = 1)).T.fillna(0.0)
# Normalization not to let this being dominated by popular genres
genres = genres / genres.sum(axis = 0)

genres = genres.T[(genres > 0).sum() > 200].T
genres = genres[genres.sum(axis = 1) > 0]
regions = regions.unstack().reset_index().set_index("id")
regions = regions.loc[regions[0] == 1, "level_0"].to_dict()
years = years.dot(years.T).unstack()
years.index.names = ("src", "trg")
years = years[years.index.get_level_values(0) != years.index.get_level_values(1)]
years = np.log(years + 1)

edges["score"] = 1 - edges["score"]
edges["score"] += edges.loc[edges["score"] > 0, "score"].min()
edges["score"] = np.log(edges["score"])
edges["score"] *= -1
edges["score"] /= edges["score"].max()

edges = edges[edges["src"].isin(regions) & edges["trg"].isin(regions) & edges["src"].isin(genres.index) & edges["trg"].isin(genres.index)]

nodes = list(set(edges["src"]) | set(edges["trg"]))
positive_sample = set(map(tuple, list(pd.concat([edges[["src", "trg"]], edges[["src", "trg"]].rename(columns = {"src": "trg", "trg": "src"})]).values)))
negative_sample = set([tuple(sorted(random.sample(nodes, 2))) for _ in range(len(positive_sample))])
negative_sample = list(negative_sample)[:edges.shape[0]]
negative_sample = pd.DataFrame(data = [(x[0], x[1], 0, 0) for x in negative_sample], columns = edges.columns)
edges = pd.concat([edges, negative_sample])

edges["exists"] = (edges["nij"] > 0).astype(int)
edges["genre_sim"] = edges.apply(lambda x: 1 - cosine(genres.loc[x["src"]], genres.loc[x["trg"]]), axis = 1)
edges["region_sim"] = edges.apply(lambda x: regions[x["src"]] == regions[x["trg"]], axis = 1).astype(int)
edges["year_sim"] = edges.apply(lambda x: years.loc[x["src"], x["trg"]], axis = 1)

edges.drop(["src", "trg"], axis = 1).to_csv("../outputs/tab7_source.tsv", sep = "\t", index = False)

